<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>Building a Completely Normal Server</title>
    <link href="/style.css" rel="stylesheet" type="text/css" media="all">
    <link href="/blog/comment/comments.css" rel="stylesheet" type="text/css" media="all">
    <!--style>
      h1 {
        background-image: url('ai_image.png');
      }
    </style-->
  </head>
  <body>
    <h1>blog</h1>
    <h2 id="caption">Building a Completely Normal Server</h2>
    <nav>
      <a href="/">home</a>
      <a href="/blog">blog</a>
    </nav>
    <?php include("../post_dates.php"); ?>
    <p>I spent the past few months making full use of all my physical engineering talents to build a NAS that is completely normal, and not weird at all, and involved an amount of 3D modeling, 3D printing, and hacksawing that is entirely typical of most home computer builds, all in the name of reusing an old laptop.</p>
    <h2>The Goal</h2>
    <p>My previous daily driver computer and current home server is a mid-range gaming laptop from late 2019 that, due to a flat lithium battery and a broken trackpad resulting from a botched storage upgrade, has been rendered more or less useless as an actual portable computer. Boasting an Intel i7-9750H, an Nvidia GTX 1660-Ti, and 16 gigabytes of aftermarket DDR4, this baby has plenty enough power to play Elite Dangerous at a buttery smooth ~90FPS, while generating enough heat to add that extra layer of immersion while flying dangerously close to main-sequence stars to harvest hydrogen from their outer atmospheres. It's also good for Minecraft servers.</p>
    <p>Now that it's in use as a home server, I find that the in-built 256 gigs of solid-state storage isn't really enough for my purposes. Now, it's plenty for one or two game servers, which is what I was using it for, but most other applications such as backups or media servers (which I would quite like to have) would be a bit stifled by that amount of space. Hence, a significant storage upgrade is in order.</p>
    <h2>The Method</h2>
    <p>There's a few ways I could go about this. My laptop actually has a decent amount of upgradeability, with a full-sized NVMe drive slot and a 2.5" SATA drive bay. The simplest, least janky, and most boring solution here would be to buy two high-capacity SSDs, throw 'em in there and be done.</p>
    <p>One problem: At over ten cents per gigabyte, those suckers aren't cheap. For this project to make sense, I'd want at least enough storage to back up my whole personal computer, plus a few terabytes of media and other miscellanies. Ideally, I'd like to never have to upgrade my storage again. Laptop-grade drives of that kind of size currently weigh in at about a thousand dollars apiece. If you look in the right places, spinning rust can be had for literally a tenth the cost per terabyte, so the choice is obvious.</p>
    <p>Next option: USB external drives. External HDD solutions such as the WD MyBook are cheaper than SSDs and are likely to be very reliable, being designed to be treated as an external peripheral and possibly moved around quite a lot. I could achieve my desired storage capacity by buying multiple of these and connecting them to my laptop.</p>
    <p>Issue the first with this idea is that these hard drives take more power in a weirder voltage configuration than a USB port is equipped to deliver, so unless I want to teach myself electrical engineering each one will require its own power cable, which after the first drive gets sketchy fast:</p>
    <img src="media/power_spaghet.png" alt="Hooking up three hard drives with this configuration means a whopping FOUR power cables for one server. Unacceptable." />
    <p>Issue the second is that USB is just not the right tool for the job here. USB is designed for hot-plugging peripherals, not semi-permanent internal aspects of a hardware configuration, so trying to pull any fancy shit like RAID or even just relying on all the drives being <em>recognized</em> every boot opens the door to all sorts of weird, unfixable problems. In other words, it's very janky, and I'd rather just not deal with all that.</p>
    <p>Of course, the reasonable way to build a NAS would be to get a machine that's actually fit for that purpose, like an off-the-shelf NAS box or a custom computer in a case with a lot of drive bays or something. I'd also rather not do this. I mean, I already have all this perfectly good computing power right here! I can't just let it go to waste. What I actually did is as follows:</p>
    <h2>The Madness</h2>
    <p>I decided to buy internal PC hard drives and hook them up to the laptop using internal connectors. This would avoid the reliability concerns of USB, and allow me to use off-the-shelf components to power all the drives off of one AC outlet, at a much cheaper price per terabyte than either of the above options. Given most laptops aren't really equipped with the requisite hardware to make such an upgrade, this presented three major problems that I was about to have a lot of fun solving.</p>
    <h3>Problem the First: Data Connections</h3>
    <p>Generally, adding internal HDDs to a computer requires the use of an internal SATA connector. My laptop has one, and I will be using it, but I'd quite like to have more than one of these drives, both for redundancy and raw capacity, so I need a way to add more. In a desktop PC, such expansion would typically be done using a spare PCIe slot on the motherboard:</p>
    <picture>
      <source srcset="media/pcie_slots.jxl" type="image/jxl">
      <img src="media/pcie_slots.jpg" alt='Three PCIe slots on a desktop PC motherboard.'>
    </picture>
    <p>However, due to the industry's blind rush to make all electronics as thin as <span title="This is a joke. PCIe expansion cards are genuinely too bulky to fit in anything with portability as a design goal. However, despite Tim Cook's best efforts, this is not true of most other expansion options.">possible</span>, no laptop on Earth has these. The meager modularity present inside laptops is typically found in the form of M.2 slots:</p>
    <picture>
      <source srcset="media/m2_slot.jxl" type="image/jxl">
      <img src="media/m2_slot.jpg" alt='An M.2 slot in a laptop, intended for an SSD.'>
    </picture>
    <p>Normally, these are used in computers of all sizes to attach NVMe SSDs, but the secret to the speed advantage of the NVMe over your standard SATA is a fundamentally different protocol: NVMe SSDs communicate directly over PCIe, meaning any NVMe slot is essentially just a tiny PCIe slot, and other types of device can take advantage of that connection, such as the SATA expander that I'll need for this project.</p>
    <p>Most laptops, including mine, have two M.2 slots: one for an SSD, and one for a wifi and bluetooth module. You can get SATA expanders for both, such as these two that I bought:</p>
    <div style="display: flex;">
      <picture style="width: 50%;">
        <source srcset="media/ChenYang.jxl" type="image/jxl">
        <img src="media/ChenYang.jpg" alt='A 2-port M.2 SATA controller, keyed for a WiFi card slot.'>
      </picture>
      <picture style="width: 50%;">
        <source srcset="media/ECS07.jxl" type="image/jxl">
        <img src="media/ECS07.jpg" alt='A 5-port M.2 SATA controller, keyed for an NVMe drive slot.'>
      </picture>
    </div>
    <p>Unfortunately, my laptop didn't want to turn on with a SATA expander in the NVMe SSD slot, so I'm stuck with the dinky two-port one. That's fine, since my server doesn't really need wireless connectivity anyway, and I've got a secret nasty trick up my sleeve that's gonna let me <em>really</em> get the most out of those two SATA ports.</p>
    <p>So, SATA as a protocol is designed to have exactly one device connected per SATA port, as opposed to USB, which supports hubs that let you turn one port into several and connect multiple devices. However, that's also not actually true at all. For reasons that are honestly beyond my understanding, some SATA controllers, such as the one on my little M.2 module, support devices that essentially do for SATA what hubs do for USB, allowing me to connect multiple drives per port.</p>
    <p>This feels a little like forbidden dark magics to me, and it probably isn't great for performance, but it lets me connect more drives so I'm using it. Keep in mind, all this is to reach a level of connectivity that almost matches what the motherboard in my desktop computer supports out of the box.</p>
    <p>Don't mind the fact that I had to cut holes in the bottom of my laptop to hook stuff up to the SATA ports. Just, um, ignore that.</p>
    <h3>Problem the Second: Powering All This Crap</h3>
    <p>Now that I've got my laptop <em>talking</em> to all these drives, they need power. I'm sure I could get this from the laptop somehow, but I'm not an electrical engineer, and the point of this project is to <em>not</em> have to buy a new computer, so I'll have to get it from elsewhere.</p>
    <p>The best way I'm aware of to power multiple hard drives from the same source is a PC power supply, such as the one I'm currently using to power the drives in my desktop PC. In fact, as part of early testing I actually had to power some drives off of my desktop PC while they were connected to the laptop. I wish I had pictures to show you of that, it was so fucked.</p>
    <p>Trouble is, PC PSUs rely on the motherboard for control. Specifically, one of the contacts in the 24-pin motherboard connector needs to be shorted to ground for the PSU to actually "S" any "P". I could do this manually with some wire, but that has the potential to be unreliable and also maybe a fire hazard. Enter this thing:</p>
    <picture>
      <source srcset="media/CSE-PTJBOD-CB1.jxl" type="image/jxl">
      <img src="media/CSE-PTJBOD-CB1.jpg" alt='A small circuit board with an ATX motherboard power connector and headers for a fan and a power button.'>
    </picture>
    <p>This is the Supermicro CSE-PTJBOD-CB1. It's a part from a Supermicro JBOD enclosure, and it exists to solve my exact problem. This little board's entire purpose is to interface a power button (and some fans) with a PC PSU, which is exactly what I need. Lucky me!</p>
    <p>Pictured above is the exact board mentioned in the <a href="https://www.servethehome.com/supermicro-cse-ptjbod-cb1-jbod-power-board-diy-jbod-chassis-made-easy/">Serve The Home article</a> I learned about this from, but this specific model appears to no longer be available, so I'll be using the more fully-featured CSE-PTJBOD-CB<em>2</em>. The only difference that matters for me is that it has headers for more fans and a power button LED.</p>
    <p>With power and storage solved, I could now confirm that this crazy plan was actually going to work. Here's a video I recorded documenting this historic moment:</p>
    <video controls>
      <source src="media/2023-11-20_contraption.mp4" type="video/mp4">
      Your browser does not support the video tag.
    </video>
    <p>And furthermore, photos of the SATA splitter working:</p>
    <div style="display: flex;">
      <picture style="width: 64%;">
        <source srcset="media/drive_spaghet.jxl" type="image/jxl">
        <img src="media/drive_spaghet.jpg" alt='Five SATA drives plugged into a laptop, one via an internal SATA bay, and the other four via an M.2 SATA controller in place of the WiFi card and a SATA splitter board. The latter four hard drives are powered off of an ATX PC PSU controlled by a Supermicro JBOD controller board.'>
      </picture>
      <picture style="width: 36%;">
        <source srcset="media/drive_spaghet_lsblk.jxl" type="image/jxl">
        <img src="media/drive_spaghet_lsblk.jpg" alt='lsblk output showing five SATA drives recognized by the system (sda through sde)'>
      </picture>
    </div>
    <h3>Problem the Third: A Case</h3>
    <p>I could run this whole setup naked, strewn out across my desk, but I'm not doing that, so I'll need a case. Unfortunately, they don't make computer cases designed for this use case. I shall have to build my own.</p>
    <p>There's a few different ways I could go about this. I've seen people make custom computer cases out of wood. <a href="https://www.youtube.com/watch?v=xE4_jKlX-xc">This guy</a> modified a rack-mount server chassis to accept his laptop's motherboard for a similar project. I don't have much in the way of woodworking ability and I'm not planning to disassemble my laptop for mounting inside a computer case, but I do have a 3D printer.</p>
    <p>I could 3D print the whole case, but that seems like it would take a lot of plastic, and more importantly, be a pain in the ass to design, especially since my 3D printer is smaller than the case I'll need to make so I can't really print it all in one go. My plan is to make the main structure out of metal rods, and 3D print various brackets to attach everything together.</p>
    <p>I originally wanted to order the rods custom-cut to the lengths I'd need, but on top of being kind of expensive the shipping would actually cost <em>more</em> than the rods themselves. In other words, too expensive. Fortunately for me, it turns out hardware stores sell lengths of steel and aluminum rods, the latter of which I can easily cut at home with a hand saw.</p>
    <p>So I bought 12 feet of aluminum rods, manually cut them to length, and then 3D printed little corner pieces out of PETG (in translucent green to match the accents on my gaming-ass laptop) to join them into a frame:</p>
    <picture>
      <source srcset="media/bare_frame.jxl" type="image/jxl">
      <img src="media/bare_frame.jpg" alt='A frame made of aluminum rods press-fit into custom 3d-printed joints'>
    </picture>
    <p>Now I need a way to mount hard drives to this. I considered 3D printing a rack of some kind for them, but I don't know how hot these things get and for obvious reasons I honestly just don't want to take any chances with them. For this reason, I instead bought a drive cage designed to mount to a couple of 5.25" drive bays, and mounted that to my frame. In fact, the height of the frame was chosen based on the size of this part.</p>
    <p>This cage also accepts a fan, so I figured I'd add a couple of those to the case, too.</p>
    <p>I attached everything to the frame using various simple 3D printed brackets I designed that just snap onto the frame, and then the various parts can be screwed into them. I specifically designed it all in order to reduce the amount of precision that was required of me.</p>
    <div style="display: flex;">
      <picture style="width: 50%;">
        <source srcset="media/partially_assembled_1.jxl" type="image/jxl">
        <img src="media/partially_assembled_1.jpg" alt='Frame made from aluminum rods and 3d-printed joints, with the following mounted to it via 3d-printed brackets: two PCBs, an ATX PSU, a power button, and a computer fan'>
      </picture>
      <picture style="width: 50%;">
        <source srcset="media/partially_assembled_2.jxl" type="image/jxl">
        <img src="media/partially_assembled_2.jpg" alt='Same frame, but from a different angle'>
      </picture>
    </div>
    <p>I wanted to make some kind of fancy bespoke mount for the laptop, but that turned out to require a lot of precise measurement and modeling that I wasn't prepared to do, so I just put a normal laptop stand on top of this thing.</p>
    <picture>
      <source srcset="media/fully_assembled.jxl" type="image/jxl">
      <img src="media/fully_assembled.jpg" alt='The frame with a drive cage installed and a running laptop sitting on top'>
    </picture>
    <p>IT'S ALIIIIIVE!!!</p>
    <h2>Software</h2>
    <p>This post is mostly about my fucked up hardware, but I figured I should share my software setup too. It's TrueNAS. I went with it because it boasts a lot of fancy features and smooths out a lot of processes that I was intimidated by the idea of doing manually in the terminal. This is probably the most normal part of the build by far, and I think some of the jankiness of the hardware side has infected it. I tried to run Syncthing on it and it just... didn't take. I don't know how else to describe it. TrueNAS is pretty good though.</p>
    <h2>Postmortem</h2>
    <p>As of writing, I've been running this thing for a couple months now, and it's been pretty good. I haven't done a lot with it yet beyond some basic backups and a Jellyfin server that I have yet to make accessible from outside my LAN, but it's been mostly reliable so far. It even survived a power outage once with no issue, though I'm currently looking into UPS solutions to make sure that doesn't happen again. All that isn't to say I'm completely happy with it, though.</p>
    <p>Firstly, after I built the chassis I discovered that you can get v-slot aluminum extrustion in custom sizes meant for exactly this type of project. They call it <a href="https://www.makerbeam.com/">MakerBeam</a>. If I had to pull some bullshit like this again I would have probably used this stuff instead of hardware store aluminum and a hacksaw.</p>
    <p>I also didn't do a perfect job of designing the 3D-printed parts. I think they came out pretty good all things considered, but I made all the brackets just a little too short and they're all a little bit bowed. Fortunately I used PETG so it just looks sketchy - PLA would have snapped by now, which I know because my first PLA test print snapped instantly!</p>
    <p>The PSU I used, a Seasonic S12III, is also a bit dubious. I picked it because it was cheap and my other Seasonic PSU is working well, and it's worked out fine so far, especially because it's a 500 watt unit which is kind of extravagant overkill for my use case, but after I bought it I learned it's actually one of the shittier units Seasonic sells. Apparently they outsource the manufacturing to some other company, which strikes me as weird considering a bunch of other companies outsource PSU manufacturing <em>to</em> Seasonic. At any rate, it's been fine so far and I don't expect it to blow up running at a fraction of its rated wattage.</p>
    <p>I got the hard drives from a website called <a href="https://serverpartdeals.com/">ServerPartDeals.com</a> that sells recertified high-capacity HDDs for like half of what they'd sell for new, and while a few months of use isn't enough to say for sure whether that was a good idea, they've certainly left a good impression. The drives I bought arrived encased in foam, inside a box which was itself packed inside another padded box. With that level of care on the part of the seller, I'm pretty comfortable trusting these drives with what I need them for.</p>
    <p>And now the elephant in the room: I cannot in good conscience recommend you do this. Having a NAS is a good idea, and using an old laptop as a basic server is a good idea, but if you're going to build a NAS like this then you should really procure a purpose-built machine, or at the very least an old desktop that can accept some extra drives. Definitely, <em>definitely</em> don't <em>buy a new laptop</em> with the intent to do this.</p>
    <p>In order to make this work I had to replace my laptop's wifi card with two layers of janky, probably AliExpress-tier hardware that is, frankly, probably introducing performance bottlenecks, or if I'm lucky, bottlenecked by the relatively low-speed interconnect of the wifi card M.2 slot. Bear in mind, that slot needs <em>at most</em> 1 gigabit of bandwidth to fulfill its intended purpose, which is about a sixth of what a single SATA 3 port can do. According to my research, that thing has two <em>separate</em> PCIe Gen 2 x 1 interfaces. Who knows what my cheap SATA controller is even doing with that.</p>
    <p>Plus, interfacing with this thing makes it painfully obvious that it was <em>not</em> designed with this kind of bullshit in mind. I ran into all kinds of Laptop Bios Moment&trade;s during this project, from the CMOS resetting every time I made a hardware change to the machine refusing to even POST with anything but an NVMe SSD in the NVMe SSD slot. I was gonna put connectors for FIVE hard drives in there. I wouldn't have even needed the SATA splitters.</p>
    <p>Honestly, I only did this at all because I figured adapting this thing to this use case would be cheaper than procuring a new machine that was as fast and I didn't want it going to waste. All that said, I have a NAS now, and it's good enough for me, and this whole adventure was a hell of a Learning Experience&trade; so I really can't complain, though if you gave me the amount of money all this stuff would have cost new to put toward a new NAS, I would have done something else.</p>
    <?php include("../../comment/form.php"); ?>
  </body>
</html>